﻿using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.IO;
using System.Linq;
using System.Text;
using EFTracingProvider;



namespace QueryProfilingExamples
{
  class Program
  {
    static void Main(string[] args)
    {
      //ToTraceString();
      //Intellitrace();
      //TracingProfiler();
      EFProf();
     // HuagatiProfiling();  //comment out the DBProviderFactores in app.config
                             //to run this. You'll need to skip the TracingProfiler
                             //method (which will fail because of the change
                             //to the app.config file
    }

    private static void HuagatiProfiling()
    {
      //you will need to comment out the DbProviderFactories
      //in app.config (they are there for EFTracing) in order for this to work
      using (var context = new AWEntities(true))
      {
        var query = from c in context.Customers select c;
        var custList = query.Take(10).ToList(); //execute query
        Customer custFirst = custList[0];
        int orderCount = custFirst.Orders.Count; //lazy load
        context.ContextOptions.LazyLoadingEnabled = false;
        Customer custSecond = custList[4];
        custSecond.Orders.Load(); //explicit load
        custSecond.ModifiedDate = DateTime.Now;
        context.SaveChanges(); //update

        //call function (stored procedure) 
        ObjectResult<PartialOrderDetails> orders =
          context.GetCustomerOrdersForId(custList[2].CustomerID);
      }
    }

    private static void EFProf()
    {
      HibernatingRhinos.Profiler.Appender.EntityFramework.EntityFrameworkProfiler.Initialize();
      using (var context = new AWEntities())
      {

        var query = from c in context.Customers select c;
        var custList = query.Take(10).ToList(); //execute query
        Customer custFirst = custList[0];
        int orderCount = custFirst.Orders.Count; //lazy load
        context.ContextOptions.LazyLoadingEnabled = false;
        Customer custSecond = custList[4];
        custSecond.Orders.Load(); //explicit load
        custSecond.ModifiedDate = DateTime.Now;
        context.SaveChanges(); //update

        //call function (stored procedure) 
        ObjectResult<PartialOrderDetails> orders =
          context.GetCustomerOrdersForId(custList[2].CustomerID);


      }

    }

    private static void Intellitrace()
    {
      using (var context = new AWEntities())
      {

        var query = from c in context.Customers select c;
        var custList = query.Take(10).ToList(); //execute query
        Customer custFirst = custList[0];
        int orderCount = custFirst.Orders.Count; //lazy load
        context.ContextOptions.LazyLoadingEnabled = false;
        Customer custSecond = custList[1];
        custSecond.Orders.Load(); //explicit load
        custSecond.ModifiedDate = DateTime.Now;
        context.SaveChanges(); //update

        //call function (stored procedure) 
        ObjectResult<PartialOrderDetails> orders =
          context.GetCustomerOrdersForId(custList[2].CustomerID);


      }
    }

    private static void ToTraceString()
    {
      using (var context = new AWEntities())
      {
        var query = from c in context.Customers where c.CustomerID == 3 select c;
        var objectQuery = query as System.Data.Objects.ObjectQuery;
        Console.WriteLine(objectQuery.ToTraceString());
      }
    }

    private static void TracingProfiler()
    {
      //be sure that the the DbProviderFactories
      //in app.config is NOT commented  in order for this to work
      // disable global logging to console
      EFTracingProviderConfiguration.LogToConsole = false;

      using (TextWriter logFile = File.CreateText("sqllogfile.txt"))
      {

        using (var context = new ExtendedAWEntities())
        {
          context.Log = logFile;

          var query = from c in context.Customers select c;
          var custList = query.Take(10).ToList(); //execute query
          Customer custFirst = custList[0];
          int orderCount = custFirst.Orders.Count; //lazy load
          context.ContextOptions.LazyLoadingEnabled = false;
          Customer custSecond = custList[1];
          custSecond.Orders.Load(); //explicit load
          custSecond.ModifiedDate = DateTime.Now;
          context.SaveChanges(); //update

          //call function (stored procedure) 
          ObjectResult<PartialOrderDetails> orders =
            context.GetCustomerOrdersForId(custList[2].CustomerID);
        }
      }
      Console.WriteLine("LOG FILE CONTENTS:");
      Console.WriteLine(File.ReadAllText("sqllogfile.txt"));
    }
  }
}
